@@ -3,10 +3,11 @@ use strict;
 use warnings;
 use parent 'Exporter';
 
+use JSON::PP;
 use Text::Trim qw(trim rtrim);
 use XSLoader;
 
-our $VERSION = '0.000070';
+our $VERSION = '0.000071';
 XSLoader::load( __PACKAGE__, $VERSION );
 
 our @EXPORT_OK = qw[];
@@ -111,7 +112,7 @@ engine
 
 =head1 VERSION
 
-Version 0.000070
+Version 0.000071
 
 =head1 SYNOPSIS
 
@@ -144,8 +145,6 @@ Version 0.000070
     $vm->set('function_name', sub { my @args = @_; return \@args; });
     my $returned = $vm->eval('function_name(my.object.slot)');
 
-    $vm->dispatch_function_in_event_loop('function_name');
-
     my $stats_href = $vm->get_stats();
     $vm->reset_stats();
 
@@ -249,22 +248,14 @@ instance of the class given by the second parameter.
 =head2 eval
 
 Run a piece of JavaScript code, given as a string, and return the results.
+After running the code, wait until all timers (if any) have been dispatched.
+
 
 For now the XS object will both compile and run the JavaScript code when this
 method is invoked; in the future this might be split into separate functions.
 
 Any returned values will be treated in the same way as a call to C<get>.
 
-=head2 dispatch_function_in_event_loop
-
-Run a JavaScript function inside an event loop, and wait until all timers have
-been dispatched.  The argument is the function name.
-
-If the function name is 'X', this is equivalent to running the following piece
-of JavaScript code:
-
-    setTimeout(function() { X(); }, 0);
-
 =head2 get_stats
 
 Return a hashref with the statistics gathered as a result of creating the XS
